"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
exports.__esModule = true;
var apollo_link_1 = require("apollo-link");
var apollo_utilities_1 = require("apollo-utilities");
var DIRECTIVE_NAME = 'serialize';
var documentCache = new Map();
function extractDirectiveArguments(doc, cache) {
    if (cache === void 0) { cache = documentCache; }
    if (cache.has(doc)) {
        return cache.get(doc);
    }
    apollo_utilities_1.checkDocument(doc);
    var directive = extractDirective(apollo_utilities_1.getOperationDefinitionOrDie(doc), DIRECTIVE_NAME);
    if (!directive) {
        return { doc: doc };
    }
    var argument = directive.arguments.find(function (d) { return d.name.value === 'key'; });
    if (!argument) {
        throw new Error("The @" + DIRECTIVE_NAME + " directive requires a 'key' argument");
    }
    if (argument.value.kind !== 'ListValue') {
        throw new Error("The @" + DIRECTIVE_NAME + " directive's 'key' argument must be of type List, got " + argument.kind);
    }
    var docWithoutDirective = apollo_utilities_1.cloneDeep(doc);
    var operationDefinition = apollo_utilities_1.getOperationDefinition(docWithoutDirective);
    operationDefinition.directives = operationDefinition.directives.filter(function (node) { return node.name.value !== DIRECTIVE_NAME; });
    var ret = {
        doc: apollo_utilities_1.removeDirectivesFromDocument([{ name: DIRECTIVE_NAME }], docWithoutDirective),
        args: argument.value
    };
    cache.set(doc, ret);
    return ret;
}
function extractKey(operation) {
    var serializationKey = operation.getContext().serializationKey;
    if (serializationKey) {
        return { operation: operation, key: serializationKey };
    }
    var _a = extractDirectiveArguments(operation.query), doc = _a.doc, args = _a.args;
    if (!args) {
        return { operation: operation };
    }
    var key = materializeKey(args, operation.variables);
    var newOperation = apollo_link_1.createOperation(operation.getContext(), __assign({}, operation, { query: doc }));
    return { operation: newOperation, key: key };
}
exports.extractKey = extractKey;
function extractDirective(query, directiveName) {
    return query.directives.filter(function (node) { return node.name.value === directiveName; })[0];
}
function materializeKey(argumentList, variables) {
    return JSON.stringify(argumentList.values.map(function (val) { return valueForArgument(val, variables); }));
}
exports.materializeKey = materializeKey;
function valueForArgument(value, variables) {
    if (value.kind === 'Variable') {
        return getVariableOrDie(variables, value.name.value);
    }
    if (value.kind === 'IntValue') {
        return parseInt(value.value, 10);
    }
    if (value.kind === 'FloatValue') {
        return parseFloat(value.value);
    }
    if (value.kind === 'StringValue' || value.kind === 'BooleanValue' || value.kind === 'EnumValue') {
        return value.value;
    }
    throw new Error("Argument of type " + value.kind + " is not allowed in @" + DIRECTIVE_NAME + " directive");
}
exports.valueForArgument = valueForArgument;
function getVariableOrDie(variables, name) {
    if (!variables || !(name in variables)) {
        throw new Error("No value supplied for variable $" + name + " used in @serialize key");
    }
    return variables[name];
}
exports.getVariableOrDie = getVariableOrDie;
//# sourceMappingURL=extractKey.js.map